@@ -12,13 +12,18 @@ http://www.apache.org/licenses/LICENSE-2.0
 import asyncio
 import contextlib
 import errno
-import json
 import logging
 import struct
 from pprint import pformat as pf
 from typing import Dict, Generator, Optional, Union
 
+# When support for cpython older than 3.11 is dropped
+# async_timeout can be replaced with asyncio.timeout
+from async_timeout import timeout as asyncio_timeout
+
 from .exceptions import SmartDeviceException
+from .json import dumps as json_dumps
+from .json import loads as json_loads
 
 _LOGGER = logging.getLogger(__name__)
 _NO_RETRY_ERRORS = {errno.EHOSTDOWN, errno.EHOSTUNREACH, errno.ECONNREFUSED}
@@ -32,9 +37,10 @@ class TPLinkSmartHomeProtocol:
     DEFAULT_TIMEOUT = 5
     BLOCK_SIZE = 4
 
-    def __init__(self, host: str) -> None:
+    def __init__(self, host: str, *, port: Optional[int] = None) -> None:
         """Create a protocol object."""
         self.host = host
+        self.port = port or TPLinkSmartHomeProtocol.DEFAULT_PORT
         self.reader: Optional[asyncio.StreamReader] = None
         self.writer: Optional[asyncio.StreamWriter] = None
         self.query_lock: Optional[asyncio.Lock] = None
@@ -64,7 +70,7 @@ class TPLinkSmartHomeProtocol:
             self.query_lock = asyncio.Lock()
 
         if isinstance(request, dict):
-            request = json.dumps(request)
+            request = json_dumps(request)
             assert isinstance(request, str)
 
         timeout = TPLinkSmartHomeProtocol.DEFAULT_TIMEOUT
@@ -77,8 +83,10 @@ class TPLinkSmartHomeProtocol:
         if self.writer:
             return
         self.reader = self.writer = None
-        task = asyncio.open_connection(self.host, TPLinkSmartHomeProtocol.DEFAULT_PORT)
-        self.reader, self.writer = await asyncio.wait_for(task, timeout=timeout)
+
+        task = asyncio.open_connection(self.host, self.port)
+        async with asyncio_timeout(timeout):
+            self.reader, self.writer = await task
 
     async def _execute_query(self, request: str) -> Dict:
         """Execute a query on the device and wait for the response."""
@@ -96,7 +104,7 @@ class TPLinkSmartHomeProtocol:
 
         buffer = await self.reader.readexactly(length)
         response = TPLinkSmartHomeProtocol.decrypt(buffer)
-        json_payload = json.loads(response)
+        json_payload = json_loads(response)
         if debug_log:
             _LOGGER.debug("%s << %s", self.host, pf(json_payload))
 
@@ -132,13 +140,13 @@ class TPLinkSmartHomeProtocol:
             except ConnectionRefusedError as ex:
                 await self.close()
                 raise SmartDeviceException(
-                    f"Unable to connect to the device: {self.host}: {ex}"
+                    f"Unable to connect to the device: {self.host}:{self.port}: {ex}"
                 )
             except OSError as ex:
                 await self.close()
                 if ex.errno in _NO_RETRY_ERRORS or retry >= retry_count:
                     raise SmartDeviceException(
-                        f"Unable to connect to the device: {self.host}: {ex}"
+                        f"Unable to connect to the device: {self.host}:{self.port}: {ex}"
                     )
                 continue
             except Exception as ex:
@@ -146,22 +154,21 @@ class TPLinkSmartHomeProtocol:
                 if retry >= retry_count:
                     _LOGGER.debug("Giving up on %s after %s retries", self.host, retry)
                     raise SmartDeviceException(
-                        f"Unable to connect to the device: {self.host}: {ex}"
+                        f"Unable to connect to the device: {self.host}:{self.port}: {ex}"
                     )
                 continue
 
             try:
                 assert self.reader is not None
                 assert self.writer is not None
-                return await asyncio.wait_for(
-                    self._execute_query(request), timeout=timeout
-                )
+                async with asyncio_timeout(timeout):
+                    return await self._execute_query(request)
             except Exception as ex:
                 await self.close()
                 if retry >= retry_count:
                     _LOGGER.debug("Giving up on %s after %s retries", self.host, retry)
                     raise SmartDeviceException(
-                        f"Unable to query the device {self.host}: {ex}"
+                        f"Unable to query the device {self.host}:{self.port}: {ex}"
                     ) from ex
 
                 _LOGGER.debug(
@@ -218,3 +225,13 @@ class TPLinkSmartHomeProtocol:
         return bytes(
             TPLinkSmartHomeProtocol._xor_encrypted_payload(ciphertext)
         ).decode()
+
+
+# Try to load the kasa_crypt module and if it is available
+try:
+    from kasa_crypt import decrypt, encrypt
+
+    TPLinkSmartHomeProtocol.decrypt = decrypt  # type: ignore[method-assign]
+    TPLinkSmartHomeProtocol.encrypt = encrypt  # type: ignore[method-assign]
+except ImportError:
+    pass
